from mycms.models import CMSEntries
from mycms.models import CMSPageTypes
from mycms.view_handlers.mycms_view import ViewObject

from mycms.view_handlers.mycms_view import ArticleList

from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned


import logging

logger = logging.getLogger("mycms.page_handlers")


# #####################
# We ensure that the base page types are already created.
# #####################


# Import page_types to ensure that the page_types are created in the database.

# TODO: There should be a better way of doing this only once after startup.

from mycms.view_handlers import page_types


class CategoryPage(page_types.BasePage):
    def __init__(self, page_object, request=None):
        self.page_object = page_object
        self.request = request

    @property
    def articles(self):

        """Here we load all pages that says we are their parent."""

        from django.db.models import Q

        # ######################################################################
        # This loads up all the articles that are
        # of type single_page or multipage and has the provided parent.
        #
        # We also paginate. So we need two information
        #
        # paginate_by = 10
        # page = 2

        limit, offset, page = self.get_list_params()

        # Get CMS entries starting at offset and give limit number results
        try:
            page_cmsentries = CMSEntries.objects.filter(
                (
                    Q(page_type=page_types.singlepageview_pagetype_obj)
                    | Q(page_type=page_types.multipageview_pagetype_obj)
                )
                & Q(path__parent__id=self.page_object.path.id)
                & Q(published=True)
                & Q(lists_include=True)
            )[offset : offset + limit]
        except Exception as e:
            print(e)
            print(e)
            pass
        # wrap the entries of the obj_list into their view_handler representations
        # obj_list_count  = CMSEntries.objects.filter((Q(page_type = page_types.singlepageview_pagetype_obj) | Q(page_type = page_types.multipageview_pagetype_obj)) &
        #                                     Q(path__parent__id = self.page_object.path.id) & Q(published=True) )[offset:offset+limit].count()

        all_cmsentries = CMSEntries.objects.filter(
            (
                Q(page_type=page_types.singlepageview_pagetype_obj)
                | Q(page_type=page_types.multipageview_pagetype_obj)
            )
            & Q(path__parent__id=self.page_object.path.id)
            & Q(published=True)
        )
        # need to know the total cmsentries in mycms.
        all_cmsentries_count = all_cmsentries.count()

        article_list = ArticleList(page_cmsentries, all_cmsentries_count, page, limit)

        for obj in page_cmsentries:
            # BUG: We are just adding everythign into the article_list here!
            article_list.append(ViewObject(page_object=obj))

        return article_list

    def get_categories(self):
        """Returns a list of all child categories of type: CATEGORY"""

        print("doping query")
        try:
            obj_list = CMSEntries.objects.filter(
                path__parent__id=self.page_object.id,
                page_type=self.page_object.page_type,
            )
        except Exception as e:
            print(e)

        print("done with the query")
        return obj_list

    @property
    def categories(self):
        return self.get_categories()

    def page_types(self):
        """
        Refactor me into a parent class.
        returns a list fo page_types
        """

        pagetype_objs = CMSPageTypes.objects.all()

        return pagetype_objs

    def on_create(self):
        pass

    def all_sub_articles(self):
        """
        Returns all articles under this category.
        """

        from django.db.models import Q

        obj_list = CMSEntries.objects.filter(
            (
                Q(page_type=page_types.singlepageview_pagetype_obj)
                | Q(page_type=page_types.multipageview_pagetype_obj)
            )
            & Q(path__path__startswith=self.page_object.path.path)
            & Q(published=True)
        )

        return obj_list



#!/usr/bin/env python
# coding: utf-8

"""
    python-creole
    ~~~~~~~~~~~~~


    :copyleft: 2008-2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import warnings
import inspect

from mycms.creole.py3compat import TEXT_TYPE
from mycms.creole.shared.utils import dict2string


class DocNode:
    """
    A node in the document tree for html2creole and creole2html.
    
    The Document tree would be created in the parser and used in the emitter.
    """

    def __init__(self, kind="", parent=None, content=None, attrs=[], level=None):
        self.kind = kind

        self.children = []
        self.parent = parent
        if self.parent is not None:
            self.parent.children.append(self)

        self.attrs = dict(attrs)
        if content:
            assert isinstance(content, TEXT_TYPE), (
                "Given content %r is not unicode, it's type: %s"
                % (content, type(content))
            )

        self.content = content
        self.level = level

    def get_attrs_as_string(self):
        """
        FIXME: Find a better was to do this.

        >>> node = DocNode(attrs={'foo':"bar", "no":123})
        >>> node.get_attrs_as_string()
        "foo='bar' no=123"

        >>> node = DocNode(attrs={"foo":'bar', "no":"ABC"})
        >>> node.get_attrs_as_string()
        "foo='bar' no='ABC'"
        """
        return dict2string(self.attrs)

    def __str__(self):
        return str(self.__repr__())

    def __repr__(self):
        return "<DocNode %s: %r>" % (self.kind, self.content)

    #        return "<DocNode %s (parent: %r): %r>" % (self.kind, self.parent, self.content)

    def debug(self):
        print("_" * 80)
        print("\tDocNode - debug:")
        print("str(): %s" % self)
        print("attributes:")
        for i in dir(self):
            if i.startswith("_") or i == "debug":
                continue
            print("%20s: %r" % (i, getattr(self, i, "---")))


class DebugList(list):
    def __init__(self, html2creole):
        self.html2creole = html2creole
        super(DebugList, self).__init__()

    def append(self, item):
        #        for stack_frame in inspect.stack(): print(stack_frame)

        line, method = inspect.stack()[1][2:4]
        msg = "%-8s   append: %-35r (%-15s line:%s)" % (
            self.html2creole.getpos(),
            item,
            method,
            line,
        )
        warnings.warn(msg)
        list.append(self, item)


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())



from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

import time

import logging
from django.conf import settings
from django.forms.models import model_to_dict
from django.utils.text import slugify
import simplejson as json
import hashlib
import math

from bs4 import BeautifulSoup


logger = logging.getLogger("mycms.view_handlers.ViewObject")

from .formatters import CreoleFormatter


class ArticleList(list):
    """
    This is used by CategoryPage to manage all articles. 
    Given an X amount of articles and we want to list a subset of it, for 
    example 10 articles per page, provide methods to be able to 
    get the current article, 
    """

    def __init__(self, page_cmsentries, total_entries, page, limit=2):
        super().__init__(self)

        self.limit = limit
        self.total_entries = total_entries
        self.page = page  # The current page we are on.

    @property
    def show_pagination(self):
        """
        Returns a boolean whether we should show the pagination or not. This 
        is True when limit is less than the total number of CMSEntries that
        can be shown.
        """

        if self.total_entries > self.limit:
            return True
        else:
            return False

    def total_pages(self):
        """
        Returns the total number of pages. 
        """
        return math.ceil(self.total_entries / self.limit)

    def page_range(self):
        return range(1, self.total_pages() + 1)

    def has_previous(self):
        return self.page > 1

    def has_next(self):
        total_pages = self.total_pages()
        return self.page < total_pages

    def has_other_pages(self):
        return self.has_previous() or self.has_next()

    def next_page(self):
        return self.page + 1

    def previous_page(self):
        return self.page - 1

    def debug(self):
        print("has_next: {}".format(self.has_next()))
        print("has_other_pages: {}".format(self.has_other_pages()))
        print("next_page: {}".format(self.next_page().title))


class ContentTopicsContainer(object):
    def __init__(self, title, anchor_text=None):

        self.title = title
        self._anchor_text = anchor_text

    @property
    def has_children(self):
        pass

    @property
    def children(self):
        """
        Returns a list of ContentTopics that exists below this content
        Topic
        """
        pass

    @property
    def anchor_text(self):

        return slugify(self.title)


class ViewObject(object):

    """
    A ViewObject represents a full page object. It takes care of
    coupling together the different pieces of a page such that it can
    be serialized.  The ViewObject handles the management of the
    attributes of the CMSEntry model.
    """

    def __init__(self, path=None, page_id=None, page_object=None, request=None):

        self.request = request

        if page_object:
            # A page_object is a models.CMSEntries instance.
            #
            self.path = page_object.path.path
            self._page_id = page_object.id
            self._obj = page_object

            if hasattr(page_object, "page_number"):
                self.page_number = page_object.page_number

        else:
            self.path = path
            self._page_id = page_id
            self._obj = None

        x = __import__("mycms.view_handlers")
        y = getattr(x, "view_handlers")

        # the view class take care of providing extra implementation that is
        # needed in the page. vie_class is defined as an attribute inside the
        # the database.
        #
        # we provide it the page_object and

        ViewClass = getattr(y, self.page_object.page_type.view_class)
        instance = ViewClass(self.page_object, request=self.request)
        self.view_handler = instance

    @property
    def DEBUG(self):
        return settings.DEBUG

    @property
    def FORCE_SHOW_ADVERTS(self):
        return getattr(settings, "FORCE_SHOW_ADVERTS", False)

    @property
    def SHOW_ADVERTS(self):

        # We follow the settings on DEBUG unless a flag for
        # FORCE_SHOW_ADVERTS has been set.

        if self.FORCE_SHOW_ADVERTS:
            print("FORCE_SHOW_ADVERTS ")
            return True
        elif self.DEBUG == False:
            return True
        else:
            return False

    def __getattr__(self, name):
        """Maps values to attributes.
        Only called if there *isn't* an attribute with this name
        """
        obj = self.view_handler

        try:
            value = getattr(obj, name)
            return value

        except AttributeError as e:
            return None

    @property
    def author(self):
        a = self.page_object.created_by
        if a is None:
            a = "admin"
        return a

    @property
    def page_object(self, reload=False):
        """
        Loads the database entry object. Raises PageDoesNotExist
        """

        if (self._obj is None) or (reload):
            from mycms.models import CMSEntries

            if self.path:
                if not self.path.startswith("/"):
                    self.path = "/" + self.path

                try:
                    obj = CMSEntries.objects.get(path__path=self.path)
                except CMSEntries.MultipleObjectsReturned:
                    # Something is wrong with the database. It is inconsistent
                    # and this is usually caused by having two pages with the same
                    # path. To resolve this, we take the newest one.

                    # TODO: Fix logging here too

                    objs = CMSEntries.objects.filter(path__path=self.path)
                    obj = objs[1]

            elif self.page_id:
                obj = CMSEntries.objects.get(pk=self.page_id)
            else:
                raise RuntimeError("ViewObject did not get a path or page_id.")

            self._obj = obj
            return obj

        else:
            return self._obj

    @property
    def page_id(self):
        if self._page_id is None:
            self._page_id = self.page_object.id
        return self._page_id

    @page_id.setter
    def page_id(self, value):
        self._page_id = value

    @property
    def title(self):
        """The page title"""
        return self.page_object.title

    @property
    def created_timestamp(self):
        return self.page_object.date_created

    @property
    def path_id(self):
        return self.page_object.path.id

    @property
    def path_str(self):
        p = self.page_object.path.path
        return p

    @property
    def html_content(self):
        """The html content of the page. This formats the page
        using the CreoleFormatter"""

        logger.debug("html_content entered")

        # TODO: Fix me: This loads only the first content entry.
        #      This should be updated to load by date.

        try:
            content_obj = self.page_object.content.all()[0]
        except IndexError as e:

            if settings.DEBUG:
                msg = """We did not find a content_obj so returning a fake content since DEBUG is swithed on."""
                logger.debug(msg)
                return CreoleFormatter().html(fake_content=True)
            else:
                return "Error: There is no content for this page."

        # TODO: Fix me: right now hardcoded to creole.

        # We pass the view into our custom CreoleFormatter so that the
        # custom creole markup can have access.
        _html_content = CreoleFormatter(content_obj.content, view=self).html()

        logger.debug(
            "Call to YACMSObject.html_content returns: \n {}".format(_html_content)
        )

        return _html_content

    @property
    def meta_keywords(self):
        """Returns a string list of keywords."""
        pass

    @property
    def meta_author(self):
        """Returns the author of the page."""
        pass

    @property
    def date_modified(self):
        """Date the page was modified"""
        pass

    @property
    def introduction(self):
        # We use beautifulsoup to extract the first paragraph
        html_content = self.html_content
        soup = BeautifulSoup(html_content)
        intro = soup.find("p")
        return str(intro.text)

    @property
    def template(self):

        # If self.page_object.template is empty, then we're fucked because
        # this will raise an AttributeError
        try:
            tmpl = self.page_object.template.name
        except AttributeError as e:
            # no specific template defined so we just use the default template
            tmpl = self.page_object.page_type.view_template

        """
        TODO: Fix this so that it is not forcing to
        get templates from mycms and instead set another
        directory.
        """
        if not tmpl.startswith("mycms"):
            tmpl = "mycms/" + tmpl

        return tmpl

    @property
    def data(self):
        d = model_to_dict(self.page_object, exclude="content")
        path_str = self.page_object.path.path
        d["path_str"] = path_str
        d["content"] = [x.id for x in self.page_object.content.all()]
        page_object = self.page_object

        # django model_to_dict ignores the datetime field.
        date_created = self.page_object.date_created
        date_created_epoch = int(time.mktime(date_created.timetuple()) * 1000)

        # django model_to_dict ignores the datetime field.
        date_modified = self.page_object.date_created
        date_modified_epoch = int(time.mktime(date_modified.timetuple()) * 1000)

        d["date_created_epoch"] = date_created_epoch
        d["date_modified_epoch"] = date_modified_epoch

        return d

    @property
    def json_data(self):

        try:
            value = json.dumps(self.data)
            return value
        except Exception as e:
            print("fuck", e)
            return ""

    @property
    def get_absolute_url(self):
        cms_base_path = getattr(settings, "YACMS_BASEPATH", None)

        if not cms_base_path:
            cms_base_path = "/cms"

        if not cms_base_path.endswith("/"):
            cms_base_path = cms_base_path.rstrip("/")

        # we assume here that self.path.path will always start with a /
        abs_url = "{}{}".format(cms_base_path, self.page_object.path.path)
        return abs_url

    @property
    def title_sha1sum(self):
        # TODO: fix this so that it is cached.
        hash_object = hashlib.md5(self.title.encode("utf-8"))
        return hash_object.hexdigest()

    @property
    def request(self):
        return self._request

    @request.setter
    def request(self, value):
        self._request = value

    @property
    # ----------------------------------------------------------------------
    def slug(self):
        """"""
        return self.page_object.slug

    # ----------------------------------------------------------------------
    def timestamp(self):
        """"""
        pass

    # ----------------------------------------------------------------------
    def created_timestamp_str(self):
        """"""
        return self.page_object.date_created.strftime("%Y/%m/%d %H:%M")

    # ----------------------------------------------------------------------
    def modified_timestamp_str(self):
        """"""
        return self.page_object.date_modified.strftime("%Y/%m/%d %H:%M")

    # ----------------------------------------------------------------------
    def id(self):
        """
        Returns the id of the curren cms_entry
        """
        _id = self.page_object.id
        return _id

    def parent_entries_list(self):

        return self.page_object.parents_list()

    def content_topics(self):
        # Get all H1 entries from the page.
        soup = BeautifulSoup(self.html_content)
        x = soup.findAll("h1", class_="multipage-submenu-h1")

        y = []
        for z in x:
            y.append(ContentTopicsContainer(z.text))
        return y

    # def on_create(self):
    #    pass


class MenuEntry(object):
    def __init__(self, cmspath):
        self.path = cmspath

    def entries(self):
        pass

    def expand(self):
        pass



class SinglePage(object):
    def __init__(self, page_object, request=None):

        self.request = request

    def on_create(self):
        pass

    @property
    def html_content(self):
        """The html content of the page. This formats the page
        using the CreoleFormatter"""

        logger.debug("html_content entered")

        # TODO: Fix me: This loads only the first content entry.
        #      This should be updated to load by date.

        try:
            content_obj = self.page_object.content.all()[0]
        except IndexError as e:

            if settings.DEBUG:
                msg = """We did not find a content_obj so returning a fake content since DEBUG is swithed on."""
                logger.debug(msg)
                return CreoleFormatter().html(fake_content=True)
            else:
                return "Error: There is no content for this page."

        # TODO: Fix me: right now hardcoded to creole.

        # We pass the view into our custom CreoleFormatter so that the
        # custom creole markup can have access.
        _html_content = CreoleFormatter(content_obj.content, view=self).html()

        logger.debug(
            "Call to YACMSObject.html_content returns: \n {}".format(_html_content)
        )

        return _html_content



#!/usr/bin/env python
# coding: utf-8

"""
    A clean reStructuredText html writer
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    It will produce a minimal set of html output.
    (No extry divs, classes oder ids.)
    
    Some code stolen from:
    http://www.arnebrodowski.de/blog/write-your-own-restructuredtext-writer.html
    https://github.com/alex-morega/docutils-plainhtml/blob/master/plain_html_writer.py
    
    :copyleft: 2011-2013 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

# import warnings
import sys

from mycms.creole.exceptions import DocutilsImportError
from mycms.creole.py3compat import TEXT_TYPE, PY3

try:
    import docutils
    from docutils.core import publish_parts
    from docutils.writers import html4css1
except ImportError:
    etype, evalue, etb = sys.exc_info()
    msg = (
        "%s - You can't use rest2html!"
        " Please install: http://pypi.python.org/pypi/docutils"
    ) % evalue
    evalue = etype(msg)

    # Doesn't work with Python 3:
    # http://www.python-forum.de/viewtopic.php?f=1&t=27507
    # raise DocutilsImportError, evalue, etb

    raise DocutilsImportError(msg)


DEBUG = False
# DEBUG = True

IGNORE_ATTR = (
    "start",
    "class",
    "frame",
    "rules",
)
IGNORE_TAGS = ("div",)


class CleanHTMLWriter(html4css1.Writer):
    """
    This docutils writer will use the CleanHTMLTranslator class below.
    """

    def __init__(self):
        html4css1.Writer.__init__(self)
        self.translator_class = CleanHTMLTranslator


class CleanHTMLTranslator(html4css1.HTMLTranslator, object):
    """
    Clean html translator for docutils system.
    """

    def _do_nothing(self, node, *args, **kwargs):
        pass

    def starttag(self, node, tagname, suffix="\n", empty=0, **attributes):
        """
        create start tag with the filter IGNORE_TAGS and IGNORE_ATTR.
        """
        #        return super(CleanHTMLTranslator, self).starttag(node, tagname, suffix, empty, **attributes)
        #        return "XXX%r" % tagname

        if tagname in IGNORE_TAGS:
            if DEBUG:
                print("ignore tag %r" % tagname)
            return ""

        parts = [tagname]
        for name, value in sorted(attributes.items()):
            # value=None was used for boolean attributes without
            # value, but this isn't supported by XHTML.
            assert value is not None

            name = name.lower()

            if name in IGNORE_ATTR:
                continue

            if isinstance(value, list):
                value = " ".join([TEXT_TYPE(x) for x in value])

            part = '%s="%s"' % (name.lower(), self.attval(TEXT_TYPE(value)))
            parts.append(part)

        if DEBUG:
            print(
                "Tag %r - ids: %r - attributes: %r - parts: %r"
                % (tagname, getattr(node, "ids", "-"), attributes, parts)
            )

        if empty:
            infix = " /"
        else:
            infix = ""
        html = "<%s%s>%s" % (" ".join(parts), infix, suffix)
        if DEBUG:
            print("startag html: %r" % html)
        return html

    def visit_section(self, node):
        self.section_level += 1

    def depart_section(self, node):
        self.section_level -= 1

    set_class_on_child = _do_nothing
    set_first_last = _do_nothing

    # remove <blockquote> (e.g. in nested lists)
    visit_block_quote = _do_nothing
    depart_block_quote = _do_nothing

    # set only html_body, we used in rest2html() and don't surround it with <div>
    def depart_document(self, node):
        self.html_body.extend(
            self.body_prefix[1:]
            + self.body_pre_docinfo
            + self.docinfo
            + self.body
            + self.body_suffix[:-1]
        )
        assert not self.context, "len(context) = %s" % len(self.context)

    # __________________________________________________________________________
    # Clean table:

    visit_thead = _do_nothing
    depart_thead = _do_nothing
    visit_tbody = _do_nothing
    depart_tbody = _do_nothing

    def visit_table(self, node):
        if docutils.__version__ > "0.10":
            self.context.append(self.compact_p)
            self.compact_p = True
        self.body.append(self.starttag(node, "table"))

    def visit_tgroup(self, node):
        node.stubs = []

    def visit_field_list(self, node):
        super(CleanHTMLTranslator, self).visit_field_list(node)
        if "<col" in self.body[-1]:
            del self.body[-1]

    def depart_field_list(self, node):
        self.body.append("</table>\n")
        self.compact_field_list, self.compact_p = self.context.pop()

    def visit_docinfo(self, node):
        self.body.append(self.starttag(node, "table"))

    def depart_docinfo(self, node):
        self.body.append("</table>\n")

    # __________________________________________________________________________
    # Clean image:

    depart_figure = _do_nothing

    def visit_image(self, node):
        super(CleanHTMLTranslator, self).visit_image(node)
        if self.body[-1].startswith("<img"):
            align = None

            if "align" in node:
                # image with alignment
                align = node["align"]

            elif node.parent.tagname == "figure" and "align" in node.parent:
                # figure with alignment
                align = node.parent["align"]

            if align:
                self.body[-1] = self.body[-1].replace(" />", ' align="%s" />' % align)


def rest2html(content, enable_exit_status=None, **kwargs):
    """
    Convert reStructuredText markup to clean html code: No extra div, class or ids.
    
    >>> rest2html("- bullet list")
    '<ul>\\n<li>bullet list</li>\\n</ul>\\n'
    
    >>> rest2html("A ReSt link to `PyLucid CMS <http://www.pylucid.org>`_ :)")
    '<p>A ReSt link to <a href="http://www.pylucid.org">PyLucid CMS</a> :)</p>\\n'
         
    >>> rest2html("========", enable_exit_status=1, traceback=False, exit_status_level=2)
    Traceback (most recent call last):
    ...
    SystemExit: 13
    """
    if not PY3:
        content = unicode(content)

    assert isinstance(
        content, TEXT_TYPE
    ), "rest2html content must be %s, but it's %s" % (TEXT_TYPE, type(content))

    settings_overrides = {
        "input_encoding": "unicode",
        "doctitle_xform": False,
        "file_insertion_enabled": False,
        "raw_enabled": False,
    }
    settings_overrides.update(kwargs)

    parts = publish_parts(
        source=content,
        writer=CleanHTMLWriter(),
        settings_overrides=settings_overrides,
        enable_exit_status=enable_exit_status,
    )
    #    import pprint
    #    pprint.pprint(parts)
    return parts["html_body"]  # Don't detache the first heading


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())

    #    print(rest2html(""")
    # +------------+------------+
    # | Headline 1 | Headline 2 |
    # +============+============+
    # | cell one   | cell two   |
    # +------------+------------+
    #    """)

    #    print(rest2html(""")
    #:homepage:
    #  http://code.google.com/p/python-creole/
    #
    #:sourcecode:
    #  http://github.com/jedie/python-creole
    #    """)

    print(
        rest2html(
            """
===============
Section Title 1
===============

---------------
Section Title 2
---------------

Section Title 3
===============

Section Title 4
---------------

Section Title 5
```````````````

Section Title 6
'''''''''''''''
    """
        )
    )



from django.contrib.sitemaps import Sitemap
from .models import CMSEntries


class CMSEntriesSiteMap(Sitemap):

    changefreq = "weekly"
    priority = 0.7

    def items(self):
        return CMSEntries.objects.filter(published=True)

    def lastmod(self, item):
        return item.date_modified



from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

from mycms import wingdbstub

# Imports
import unittest
import mock

from random import randint
import loremipsum


# Django Imports
from django.test import TestCase
from django.template.defaultfilters import slugify
from django.http import HttpRequest

# Local Application Imports

import mycms
from mycms import exceptions
from mycms import pageview
from mycms.models import Pages
from mycms.models import Paths
from mycms import YACMS_BASE_URL

from django.conf import settings


class ModelTests(unittest.TestCase):
    def test_can_create_default_base_path(self):
        """Ensure that we can create a base path, where entry 
        is at / and has no parent"""

        # In order to be able to do this test, we need to delete all the
        # Paths

        for entry in Pages.objects.all():
            entry.delete()

        for entry in Paths.objects.all():
            entry.delete()

        path_instance = Paths()
        path_instance.save()
        self.assertEqual(path_instance.path, "/")

    def test_can_create_root_path(self):
        """Ensure we can create a basic Path entry"""

        path_instance = Paths(path="/sysadmin")
        path_instance.save()

        retrieved_path_instance = Paths.objects.get(path="/sysadmin")
        self.assertEqual(retrieved_path_instance, path_instance)

    def test_can_create_parent_paths_recursively(self):

        path_instance = Paths(path="/foo/bar/baz")
        path_instance.save()

        foo_bar_baz = Paths.objects.get(path="/foo/bar/baz")
        foo_bar = Paths.objects.get(path="/foo/bar")
        foo = Paths.objects.get(path="/foo")

        self.assertEqual(foo_bar_baz.path, "/foo/bar/baz")
        self.assertEqual(foo_bar.path, "/foo/bar")
        self.assertEqual(foo.path, "/foo")

    def test_can_create_a_page_entry(self):
        """
        Ensure that given a path, we can create a page. This test
        provides all the required values. 
        """

        path_instance = Paths(path="/this/is/a/test/article")
        path_instance.save()

        page_instance = Pages()

        page_values = {
            "title": "This is the title",
            "content": "This is just some content.",
            "page_type": "HTMLPAGE",
            "template": "index.html",
            "frontpage": True,
            "published": True,
            "meta_description": "This is the meta description.",
            "article_logo": "fedora-logo.png",
        }

        page_instance.path = path_instance
        page_instance.title = page_values.get("title")
        page_instance.slug = slugify(page_values.get("title"))
        page_instance.content = page_values.get("content")
        page_instance.page_type = page_values.get("page_type")
        page_instance.template = page_values.get("template")
        page_instance.frontpage = page_values.get("frontpage")
        page_instance.published = page_values.get("published")
        page_instance.meta_description = page_values.get("meta_description")
        page_instance.article_logo = page_values.get("article_logo")
        page_instance.save()

        # The use cases is to load by path so we try to do the same here.
        page_instance_from_db = Pages.objects.get(path=path_instance)

        # Now ensure that everything was saved properly.
        for key in page_values.keys():
            page_value = page_values.get(key)
            db_value = getattr(page_instance_from_db, key)
            print("Comparing \t'{}' \t: \t db '{}'".format(page_value, db_value))
            self.assertEqual(page_value, db_value)

    def test_can_create_a_page_with_default_values_via_save_override(self):
        """
        This test creates a page given a minimum amount of values. 
        """

        path_instance = Paths(path="/this/is/a/test/mimimal_article")
        path_instance.save()

        page_instance = Pages()

        page_values = {
            "title": "This is the title",
            "content": "This is just some content.",
            "page_type": "HTMLPAGE",
            "template": "index.html",
            "frontpage": True,
            "published": True,
            "meta_description": "This is the meta description.",
            "article_logo": "fedora-logo.png",
        }

        page_instance.path = path_instance
        page_instance.title = page_values.get("title")
        # Do not set the slug because it should be filled automatically
        # page_instance.slug = slugify(page_values.get("title"))
        page_instance.content = page_values.get("content")
        page_instance.page_type = page_values.get("page_type")
        # Do not set the template because ave should fill it in automatically
        # page_instance.template = page_values.get("template")
        page_instance.frontpage = page_values.get("frontpage")
        page_instance.published = page_values.get("published")
        page_instance.meta_description = page_values.get("meta_description")
        page_instance.article_logo = page_values.get("article_logo")
        page_instance.save()

        print(page_instance.slug)

        self.assertEqual(page_instance.slug, slugify(page_values.get("title")))
        self.assertEqual(
            page_instance.template, "{}.html".format(page_instance.page_type.lower())
        )


class ExtendedModelTests(unittest.TestCase):
    """Test the rest of the attributes."""

    def setUp(self):
        """Create a base url"""
        self.path_inst = Paths(path="/articles/my-test-article")
        self.path_inst.save()

        page = Pages()

        page.path = self.path_inst
        page.title = "My Test Article"
        page.content = "This is just a dummy content"
        page.page_type = "HTMLVIEW"
        page.frontpage = True
        page.published = True
        page.meta_description = ""
        page.save()

        self.page_instance = page

    def test_get_absoloute_url(self):
        """Tests to show that we can get a correct absolute URL."""
        # Ensure that we can get a correct absolute URL

        # Override the settings.YACMS_BASE_URL with our value

        url = self.page_instance.get_absolute_url()
        expected_url = "/{}{}".format(
            mycms.YACMS_BASE_URL, self.page_instance.path.path.lstrip("/")
        )
        self.assertEqual(url, expected_url)

    # def test_introduction(self):
    # self.fail()

    # def test_logo(self):
    # self.fail()

    # def test_data_dict(self):
    # self.fail()

    # def test_view(self):
    # """A simple test to show that view returns a PageView. This will
    # fail if something is wrong with the pageview module which has its
    # own set of tests."""
    # self.fail()



#!/usr/bin/env python
# coding: utf-8


"""
    python-creole utils
    ~~~~~~~~~~~~~~~~~~~    


    :copyleft: 2008-2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import re


space_re = re.compile(r"^(\s*)(.*?)(\s*)$", re.DOTALL)


def clean_whitespace(txt):
    """
    Special whitespaces cleanup

    >>> clean_whitespace("\\n\\nfoo bar\\n\\n")
    'foo bar\\n'

    >>> clean_whitespace("   foo bar  \\n  \\n")
    ' foo bar\\n'

    >>> clean_whitespace(" \\n \\n  foo bar   ")
    ' foo bar '

    >>> clean_whitespace("foo   bar")
    'foo   bar'
    """

    def cleanup(match):
        start, txt, end = match.groups()

        if " " in start:
            start = " "
        else:
            start = ""

        if "\n" in end:
            end = "\n"
        elif " " in end:
            end = " "

        return start + txt + end

    return space_re.sub(cleanup, txt)


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())



# coding: utf-8

"""
    Helper to support Python v2 and v3
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Some ideas borrowed from six
    
    See also:
        http://python3porting.com
        https://bitbucket.org/gutworth/six/src/tip/six.py
        http://packages.python.org/six/
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import sys
import doctest
import re

# True if we are running on Python 3.
PY3 = sys.version_info[0] == 3


if PY3:
    TEXT_TYPE = str
    BINARY_TYPE = bytes
else:
    TEXT_TYPE = unicode
    BINARY_TYPE = str

    # Simple remove 'u' from python 2 unicode repr string
    # See also:
    # http://bugs.python.org/issue3955
    # http://www.python-forum.de/viewtopic.php?f=1&t=27509 (de)
    origin_OutputChecker = doctest.OutputChecker

    class OutputChecker2(origin_OutputChecker):
        def check_output(self, want, got, optionflags):
            got = got.replace("u'", "'").replace('u"', '"')
            return origin_OutputChecker.check_output(self, want, got, optionflags)

    doctest.OutputChecker = OutputChecker2


def repr2(obj):
    """
    Don't mark unicode strings with u in Python 2
    """
    if not PY3:
        return repr(obj).lstrip("u")
    else:
        return repr(obj)



from django import template


register = template.Library()


@register.tag
def PageAdmin(parser, token):

    return PageAdminNode()


class PageAdminNode(template.Node):
    def __init__(self):
        pass

    def render(self, context):
        # Load the template and return it
        self.request = template.Variable("request")
        self.request.jason = "monkey"
        t = template.loader.get_template("mycms/templatetags/PageAdmin.html")

        return t.render(context.flatten())



#!/usr/bin/env python
# coding: utf-8

"""
    distutils setup
    ~~~~~~~~~~~~~~~

   :copyleft: 2009-2018 by Jason Viloria , see AUTHORS.txt for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals
import os
import sys

from setuptools import setup, find_packages, Command

VERSION_STRING="0.1.9"
URL = "https://github.com/jnvilo/mycms"
DOWNLOAD_URL="{}/archive/{}.tar.gz".format(URL, VERSION_STRING)
PACKAGE_ROOT = os.path.dirname(os.path.abspath(__file__))


def get_authors():
    try:
        with open(os.path.join(PACKAGE_ROOT,"AUTHORS.txt"), "r") as f: 
            authors = [l.strip(" *\r\n") for l in f if l.strip().startswith("*")]
    except Exception:
        evalue = sys.exc_info()[1]
        authors = "[Error: %s]" % evalue
        print("Error: {}".format(evalue))
    return authors


def get_requirements():
    reqs = []
    try:
        f = open(os.path.join(PACKAGE_ROOT,"requirements.txt"), "r")
        l = f.readlines()
        for e in l:
            reqs.append(e.strip("\n"))
    except Exception:
        print("Failed to install requirements") 

    return reqs

def get_long_description():
    
    with open(os.path.join(PACKAGE_ROOT,"README.md")) as f:
        long_description = f.read()
        
    return long_description

	
    
setup(
    name='mycms',
    version=VERSION_STRING,
    description='Yet another CMS. This one for django.',
    long_description=get_long_description(), 
    long_description_content_type="text/markdown",
    author=get_authors(),
    author_email="jnvilo@gmail.com",
    maintainer="Jason Viloria",
    url=URL,
    download_url = DOWNLOAD_URL, 
    packages=find_packages(),
    install_requires=get_requirements(), 
    include_package_data=True, # include package data under svn source control
    #entry_points={
    #    "console_scripts": [
    #        "creole2html = creole.cmdline:cli_creole2html",
    #        "html2creole = creole.cmdline:cli_html2creole
    #        "html2rest = creole.cmdline:cli_html2rest",
    #        "html2textile = creole.cmdline:cli_html2textile",
    #    ],
    #},
    #zip_safe=True, # http://packages.python.org/distribute/setuptools.html#setting-the-zip-safe-flag
    keywords=["django","cms","blog"] ,
    classifiers=[
        # http://pypi.python.org/pypi?%3Aaction=list_classifiers
        #"Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Operating System :: OS Independent",
        "Topic :: Documentation",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    #test_suite="yacms.tests.get_test_suite",
)



from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import
from django.db.models.signals import post_save
from django.db.utils import OperationalError
import pathlib
from datetime import datetime
from django.core.cache import cache

from django.db import models
from django.template.defaultfilters import slugify
from django.contrib.auth.models import User
from django.conf import settings
from django.db.models.signals import post_save
from django.dispatch import receiver


from mycms.creole import creole2html


from rest_framework.authtoken.models import Token

from loremipsum import generate_paragraphs


class CMSPaths(models.Model):

    # class Meta:
    #    db_table = "yacms_cmspaths" #To remove when we no longer need to actuall db contents.

    path = models.CharField(max_length=2000, null=True)
    parent = models.ForeignKey(
        "CMSPaths", null=True, blank=True, on_delete=models.DO_NOTHING
    )

    def __str__(self):
        return self.path


class CMSTags(models.Model):

    # class Meta:
    #    db_table = "yacms_cmstags"

    name = models.CharField(max_length=256, default="NotSet")

    def __str__(self):
        return self.name


class CMSMarkUps(models.Model):
    # class Meta:
    #    db_table = "yacms_cmsmarkups"

    markup = models.CharField(max_length=128, default="Creole")

    def __str__(self):
        return self.markup


class CMSContents(models.Model):

    # class Meta:
    #    db_table = "yacms_cmscontents"

    title = models.CharField(max_length=1024, null=True, blank=True)
    content = models.TextField(max_length=20480, default="Empty")
    timestamp = models.DateTimeField(auto_now=True)
    markup = models.ForeignKey(CMSMarkUps, null=True, on_delete=models.DO_NOTHING)
    meta_description = models.TextField(max_length=20480, default="", blank=True)
    tags = models.ManyToManyField(CMSTags, blank=True)
    page = models.IntegerField(default=1)

    def __str__(self):
        return self.content

    @property
    def html(self):

        # Lazy import is needed because the mycms.view_handlers requires to import
        # mycms.models.CMSEntries, but unfortunately the models will not yet
        # be available during the django startup and causes an exception.
        from mycms.view_handlers.formatters import CreoleFormatter

        return CreoleFormatter(self.content).html()


class CMSTemplates(models.Model):

    # class Meta:
    #    db_table = "yacms_cmstemplates"

    name = models.CharField(max_length=1024, default="page.html")
    template = models.TextField(max_length=10240, default="empty template")

    def __str__(self):
        return self.name


class CMSPageTypes(models.Model):

    # class Meta:
    #    db_table = "yacms_cmspagetypes"

    page_type = models.CharField(max_length=64, default="DefaultType")
    text = models.CharField(max_length=128, default="default class")
    view_class = models.CharField(max_length=256, default="DefaultView")
    view_template = models.CharField(max_length=32, default=None)

    def save(self, *args, **kwargs):
        # if (self.pk is None) and (self.view_template is None):
        if self.view_template is None:
            # We only care to put the view template as the name of the view_class
            # during the creation.
            self.view_template = "DefaultView.html"

        super(CMSPageTypes, self).save(*args, **kwargs)

    def __str__(self):
        return self.text


def get_admin_user():

    try:
        admin = User.objects.get(username="admin")
        return admin
    except OperationalError as e:
        return 1


class CMSEntries(models.Model):

    # class Meta:
    #    db_table = "yacms_cmsentries"

    title = models.CharField(max_length=1024, default=None)
    path = models.ForeignKey(CMSPaths, null=True, on_delete=models.DO_NOTHING)
    slug = models.SlugField(max_length=1024, unique=True)

    # We make the content a many to many to be able to handle multiple
    # so we can version by published.
    content = models.ManyToManyField(CMSContents, blank=True)
    date_created = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now_add=True)

    page_type = models.ForeignKey(
        "CMSPageTypes", null=True, on_delete=models.DO_NOTHING
    )
    template = models.ForeignKey(
        CMSTemplates, null=True, blank=True, on_delete=models.DO_NOTHING
    )

    frontpage = models.BooleanField(default=False)
    published = models.BooleanField(default=False)
    lists_include = models.BooleanField(default=True)

    page_number = models.IntegerField(default=1)
    created_by = models.ForeignKey(
        User,
        # default=get_admin_user().pk,
        null=True,
        blank=True,
        on_delete=models.DO_NOTHING,
    )

    logo_url = models.CharField(
        default="/static/mycms/images/png/default.png",
        null=True,
        blank=True,
        max_length=1024,
    )

    def toggle_published(self):
        if self.published:
            self.published = False
        else:
            self.published = True

    def toggle_frontpage(self):

        if self.frontpage:
            self.frontpage = False
        else:
            self.frontpage = True

    def on_create(self):

        view_object = self.view

        if hasattr(view_object, "on_create"):
            view_object.on_create()

    def parent(self):

        # Get the cms entry that has a path belonging to the parent of our path.
        parent_entry = CMSEntries.objects.get(path=self.path.parent)
        return parent_entry

    def html_content(self):
        # We are going to index the parsed content of the CMSEntries
        # so we are going to ask our view to give that to us.

        # Fortunately the view for this CMSEntrie should alread
        # implement html_content so we let it handle it.
        return self.view.html_content

    def __str__(self):
        return self.title

    # ----------------------------------------------------------------------
    @property
    def date_created_str(self):
        """"""
        # value = self.date_created.strftime("%Y%m%d %H:%M")
        value = self.date_created
        return value

    @property
    def date_modified_str(self):
        """"""
        value = self.date_modified.strftime("%Y%m%d %H:%M")
        return value

    @property
    def view(self):
        from mycms.view_handlers import ViewObject

        view_object = ViewObject(page_object=self)
        return view_object

    @property
    def view_object(self):
        return self.view

    def get_absolute_url(self):
        cms_base_path = getattr(settings, "YACMS_BASEPATH", None)

        if not cms_base_path:
            cms_base_path = "/cms"

        if not cms_base_path.endswith("/"):
            cms_base_path = cms_base_path.rstrip("/")

        # we assume here that self.path.path will always start with a /
        return "{}{}".format(cms_base_path, self.path.path)

    def get_parent_paths(self, path_str):

        x = path_str.rfind("/")

        if x == 0:
            # we are at the root
            return [path_str]

        else:
            return self.get_parent_paths(path_str[:x]) + [path_str]

    def parents_list(self):

        path_str = self.path.path

        p = self.get_parent_paths(path_str)

        pl = []
        for path_str in p:

            pl.append(CMSEntries.objects.get(path__path=path_str))

        return pl

    def categories(self):
        c = CMSEntries.objects.filter(
            path__parent=self.path, page_type__page_type="CATEGORY", published=True
        )
        return c

    def save(self, *args, **kwargs):
        if self.pk is None:
            super(CMSEntries, self).save(*args, **kwargs)

            # Commenting out self.on_create() because we still do not have
            # a path yet and as a result we can not call self.on_create()
            # self.on_create()

        else:
            super(CMSEntries, self).save(*args, **kwargs)

    # def save(self, *args, **kwargs):

    # if self.pk is None:
    # tell parent to save ourselves so we get a pk.
    # super(CMSEntries, self).save(*args, **kwargs)

    # print("WE HAVE A PK: {} ".format(self.pk))
    ##This is a new cms entry
    ##create a new content and
    # paragraphs = generate_paragraphs(5, start_with_lorem=False)
    # p = ""
    # for paragraph in paragraphs:
    # p =  unicode(paragraph[2]) + "\n\n" + p
    # h =  creole2html(p)

    # obj = CMSContents()
    # obj.content = "This is a new content."
    # obj.save()

    # cms_obj = CMSEntries.objects.get(id=self.pk)
    # cms_obj.content.add(obj)

    # else:
    # super(CMSEntries, self).save(*args, **kwargs)


## method for updating
# def create_default_content(sender, instance, created, **kwargs):

# if created:
# obj = CMSContents()
# obj.content = "This is the latest content1"
# obj.save()

# c = CMSContents.objects.get(id=obj.id)
# instance.content.add(c)
# print(type(instance))

## register the signal
# post_save.connect(create_default_content, sender=CMSEntries, dispatch_uid="CREATE_CONTENT")


class CMSArchivesIndex(models.Model):

    month = models.IntegerField(default=0)
    year = models.IntegerField(default=1975)
    entries = models.ManyToManyField(CMSEntries, blank=True)


@receiver(post_save, sender=settings.AUTH_USER_MODEL)
def create_auth_token(sender, instance=None, created=False, **kwargs):
    if created:
        Token.objects.create(user=instance)

    #



from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

from rest_framework.renderers import JSONRenderer
from rest_framework.parsers import JSONParser
from rest_framework import serializers

from mycms.models import CMSContents
from mycms.models import CMSMarkUps
from mycms.models import CMSTemplates
from mycms.models import CMSPageTypes
from mycms.models import CMSEntries
from mycms.models import CMSPaths

import os
import random
from faker import Faker


class LoremIpsumSerializer(serializers.Serializer):

    num_paragraphs = serializers.IntegerField()

    class Meta:
        fields = ["num_paragraphs"]


class CMSPathsSerializer(serializers.ModelSerializer):
    class Meta:
        model = CMSPaths
        fields = ("id", "path", "parent")


class CMSPageTypesSerializer(serializers.ModelSerializer):
    class Meta:
        model = CMSPageTypes
        fields = ("id", "page_type", "text", "view_class", "view_template")


class CMSContentsSerializer(serializers.ModelSerializer):
    class Meta:
        model = CMSContents
        fields = ("id", "content", "timestamp", "markup", "title")


class CMSMarkUpSerializer(serializers.ModelSerializer):
    class Meta:
        model = CMSMarkUps
        fields = ("id", "markup")


class CMSTemplatesSerializer(serializers.ModelSerializer):
    class Meta:
        model = CMSTemplates
        fields = ("id", "name", "template")


class CMSEntrySerializer(serializers.ModelSerializer):

    path = serializers.StringRelatedField()
    title = serializers.CharField(max_length=1024, help_text="The title of the page.")
    # content = serializers.CharField(default="[]")
    # template = serializers.IntegerField(default=1,
    #                                    help_text="Index value, pk for template")
    frontpage = serializers.BooleanField(
        default=False, help_text="default to False. Set True to display in frontpage"
    )
    published = serializers.BooleanField(
        default=False, help_text="Published defaults to False"
    )
    page_number = serializers.IntegerField(default=1, help_text="page_number")

    # content = serializers.ManyRelatedField(child_relation=CMSContents,
    #                                       help_text="array of PK of content objects.")
    class Meta:
        model = CMSEntries
        fields = (
            "id",
            "title",
            "path",
            "slug",
            "content",
            "date_created",
            "page_type",
            "frontpage",
            "published",
            "page_number",
        )


class CMSEntryExpandedSerializer(serializers.ModelSerializer):

    path = serializers.StringRelatedField()
    template = serializers.IntegerField(
        default=1, help_text="Index value, pk for template"
    )

    class Meta:
        model = CMSEntries
        fields = (
            "id",
            "title",
            "path",
            "slug",
            "content",
            "date_created",
            "page_type",
            "template",
            "frontpage",
            "published",
            "page_number",
            "date_modified",
        )


class EntryData(object):
    def __init__(self, **kwargs):
        for field in ("id", "title", "slug", "parent"):
            setattr(self, field, kwargs.get(field, None))


class CMSChildEntrySerializer(serializers.ModelSerializer):

    template = serializers.IntegerField(required=False)
    content = CMSContentsSerializer(many=True, required=False)

    class Meta:
        model = CMSEntries

        fields = (
            "id",
            "title",
            "slug",
            "content",
            "date_created",
            "page_type",
            "template",
            "frontpage",
            "published",
            "page_number",
        )

    def make_path(self, slug, parent_id):
        """
        Creates a CMSPaths object for the current CMSEntry.
        """

        parent_obj = CMSEntries.objects.get(id=parent_id)
        path_str = os.path.join(parent_obj.path.path, slug)
        path_obj, c = CMSPaths.objects.get_or_create(
            path=path_str, parent=parent_obj.path
        )

        if not c:
            print("Warning. Recreated {}".format(path_str))
            # We need to check if there exists an article with this Path.
            entry = CMSEntries.objects.filter(path=path_obj)
            if entry:
                raise Exception(
                    "Article: {} already exists. Refuse to create. ".format(entry.title)
                )
        return path_obj

    def create(self, validated_data, parent_id=None):

        title = validated_data["title"]
        slug = validated_data["slug"]
        try:
            content = validated_data["content"]
        except KeyError as e:
            # This means no content was provided. Previous versions of
            # DRF would provide content = [] when no content was provided
            # so this is just a hack!
            content = []

        template = validated_data.get("template", None)
        published = validated_data["published"]
        frontpage = validated_data["frontpage"]
        page_type = validated_data["page_type"]

        # TODO: Currently even though we accept a path id, we do not use it
        # and instead we insist on getting or creating the CMSPaths object
        # which has the path corresponding to this CMSEntry.

        child = CMSEntries()
        child.title = title
        child.slug = slug
        child.frontpage = frontpage
        child.published = published

        # Need to call save before being able to add any reference
        child.save()

        path_obj = self.make_path(slug, parent_id)

        child.path = path_obj
        child.template = template

        if len(content) == 0:
            content_entry = CMSContents(content=self.fake_content())
            content_entry.save()
            child.content.add(content_entry)
        else:
            # TODO: This has never been tested. Test this and remove.
            child.content = content

        child.page_type = page_type
        child.save()

        # Now call the on_create() of the Child Entry

        child.on_create()

        return child

    def fake_content(self):

        fake = Faker()

        num_paragraphs = random.randint(3, 5)

        paragraphs = []
        for c in range(num_paragraphs):
            num_sentences = random.randint(5, 10)
            paragraphs.append(" ".join(fake.paragraphs(num_sentences)))

        return "\n\n".join(paragraphs)


class CMSPathFullSerializer(serializers.ModelSerializer):
    class Meta:
        model = CMSPaths

        fields = ["path", "parent"]


class CMSPageSerializer(serializers.ModelSerializer):

    path = CMSPathFullSerializer()
    content = CMSContentsSerializer(many=True)

    class Meta:
        model = CMSEntries
        title = serializers.CharField(max_length=1024, default=None)

        fields = (
            "id",
            "title",
            "slug",
            "content",
            "date_created",
            "page_type",
            "template",
            "frontpage",
            "published",
            "page_number",
            "path",
        )

    # path = serializers.Field.ForeignKey(CMSPaths,
    #                         null=True,
    #                         on_delete=models.DO_NOTHING)
    # slug = models.SlugField(max_length=1024, unique=True)

    ##We make the content a many to many to be able to handle multiple
    ##so we can version by published.
    # content = models.ManyToManyField(CMSContents, blank=True)
    # date_created = models.DateTimeField(auto_now_add=True)
    # date_modified = models.DateTimeField(auto_now_add=True)

    # page_type = models.ForeignKey("CMSPageTypes",
    # null=True,
    # on_delete=models.DO_NOTHING)
    # template = models.ForeignKey(CMSTemplates,
    # null=True,
    # blank=True,
    # on_delete=models.DO_NOTHING)

    # frontpage = models.BooleanField(default=False)
    # published = models.BooleanField(default=False)

    # page_number = models.IntegerField(default=1)
    # created_by = models.ForeignKey(User,
    ##default=get_admin_user().pk,
    # null=True,
    # blank=True,
    # on_delete=models.DO_NOTHING)


class CMSAuthTokenSerializer(serializers.Serializer):

    pass



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-06-13 12:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mycms", "0002_cmsarchivesindex"),
    ]

    operations = [
        migrations.AddField(
            model_name="cmsentries",
            name="lists_include",
            field=models.BooleanField(default=True),
        ),
    ]



# coding: utf-8


"""
    WikiCreole to HTML converter

    :copyleft: 2008-2014 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from __future__ import division, absolute_import, print_function, unicode_literals

from xml.sax.saxutils import escape
import sys
import traceback

from mycms.creole.creole2html.parser import CreoleParser
from mycms.creole.py3compat import TEXT_TYPE, repr2
from mycms.creole.shared.utils import string2dict


class TableOfContent(object):
    def __init__(self):
        self.max_depth = None
        self.headlines = []
        self._created = False
        self._current_level = 0

    def __call__(self, depth=None, **kwargs):
        """Called when if the macro <<toc>> is defined when it is emitted."""
        if self._created:
            return "&lt;&lt;toc&gt;&gt;"

        self._created = True
        if depth is not None:
            self.max_depth = depth

        return "<<toc>>"

    def add_headline(self, level, content):
        """Add the current header to the toc."""
        if self.max_depth is None or level <= self.max_depth:
            self.headlines.append((level, content))

    def flat_list2nest_list(self, flat_list):
        # this func code based on borrowed code from EyDu, Thanks!
        # http://www.python-forum.de/viewtopic.php?p=258121#p258121
        tree = []
        stack = [tree]

        for index, element in flat_list:
            stack_length = len(stack)

            if index > stack_length:
                for _ in range(stack_length, index):
                    l = []
                    stack[-1].append(l)
                    stack.append(l)
            elif index < stack_length:
                stack = stack[:index]

            stack[-1].append(element)

        return tree

    def nested_headlines2html(self, nested_headlines, level=0):
        """Convert a python nested list like the one representing the toc to an html equivalent."""
        indent = "\t" * level
        if isinstance(nested_headlines, TEXT_TYPE):
            return '%s<li><a href="#%s">%s</a></li>\n' % (
                indent,
                nested_headlines,
                nested_headlines,
            )
        elif isinstance(nested_headlines, list):
            html = "%s<ul>\n" % indent
            for elt in nested_headlines:
                html += self.nested_headlines2html(elt, level + 1)
            html += "%s</ul>" % indent
            if level > 0:
                html += "\n"
            return html

    def emit(self, document):
        """Emit the toc where the <<toc>> macro was."""
        nested_headlines = self.flat_list2nest_list(self.headlines)
        html = self.nested_headlines2html(nested_headlines)

        """
        In order to make it easier to selectively style the created toc, we sorround
        the ul with a div to be able to style the toc independently.
        
        This allows us to style the menu. 
        """

        html = """<div class="creole_macro toc"><span class="toc">{}</span></div>""".format(
            html
        )

        # FIXME: We should not use <p> here, because it doesn't match
        #        if no newline was made before <<toc>>
        if "<p><<toc>></p>" in document:
            document = document.replace("<p><<toc>></p>", html, 1)
        else:
            document = document.replace("<<toc>>", html, 1)

        return document


class HtmlEmitter(object):
    """
    Generate HTML output for the document
    tree consisting of DocNodes.
    """

    def __init__(self, root, macros=None, verbose=None, stderr=None, view=None):
        self.root = root
        self.view = view

        if callable(macros) == True:
            # was a DeprecationWarning in the past
            raise TypeError("Callable macros are not supported anymore!")

        if macros is None:
            self.macros = {}
        else:
            self.macros = macros

        if not "toc" in root.used_macros:
            # The document has no <<toc>>
            self.toc = None
        else:
            if isinstance(self.macros, dict):
                if "toc" in self.macros:
                    self.toc = self.macros["toc"]
                else:
                    self.toc = TableOfContent()
                    self.macros["toc"] = self.toc
            else:
                try:
                    self.toc = getattr(self.macros, "toc")
                except AttributeError:
                    self.toc = TableOfContent()
                    self.macros.toc = self.toc

        if verbose is None:
            self.verbose = 1
        else:
            self.verbose = verbose

        if stderr is None:
            self.stderr = sys.stderr
        else:
            self.stderr = stderr

    def get_text(self, node):
        """Try to emit whatever text is in the node."""
        try:
            return node.children[0].content or ""
        except:
            return node.content or ""

    def html_escape(self, text):
        return escape(text)

    def attr_escape(self, text):
        return self.html_escape(text).replace('"', "&quot")

    # *_emit methods for emitting nodes of the document:

    def document_emit(self, node):
        return self.emit_children(node)

    def text_emit(self, node):
        return self.html_escape(node.content)

    def separator_emit(self, node):
        return "<hr />\n\n"

    def paragraph_emit(self, node):
        return "<p>%s</p>\n" % self.emit_children(node)

    def _list_emit(self, node, list_type):
        if node.parent.kind in ("document",):
            # The first list item
            formatter = ""
        else:
            formatter = "\n"

        if list_type == "li":
            formatter += "%(i)s<%(t)s>%(c)s</%(t)s>"
        else:
            formatter += "%(i)s<%(t)s>%(c)s\n" "%(i)s</%(t)s>"
        return formatter % {
            "i": "\t" * node.level,
            "c": self.emit_children(node),
            "t": list_type,
        }

    def bullet_list_emit(self, node):
        return self._list_emit(node, list_type="ul")

    def number_list_emit(self, node):
        return self._list_emit(node, list_type="ol")

    def list_item_emit(self, node):
        return self._list_emit(node, list_type="li")

    def table_emit(self, node):
        return "<table>\n%s</table>\n" % self.emit_children(node)

    def table_row_emit(self, node):
        return "<tr>\n%s</tr>\n" % self.emit_children(node)

    def table_cell_emit(self, node):
        return "\t<td>%s</td>\n" % self.emit_children(node)

    def table_head_emit(self, node):
        return "\t<th>%s</th>\n" % self.emit_children(node)

    # --------------------------------------------------------------------------

    def _typeface(self, node, tag):
        return "<%(tag)s>%(data)s</%(tag)s>" % {
            "tag": tag,
            "data": self.emit_children(node),
        }

    # TODO: How can we generalize that:
    def emphasis_emit(self, node):
        return self._typeface(node, tag="i")

    def strong_emit(self, node):
        return self._typeface(node, tag="strong")

    def monospace_emit(self, node):
        return self._typeface(node, tag="tt")

    def superscript_emit(self, node):
        return self._typeface(node, tag="sup")

    def subscript_emit(self, node):
        return self._typeface(node, tag="sub")

    def underline_emit(self, node):
        return self._typeface(node, tag="u")

    def small_emit(self, node):
        return self._typeface(node, tag="small")

    def delete_emit(self, node):
        return self._typeface(node, tag="del")

    # --------------------------------------------------------------------------

    def header_emit(self, node):
        header = "<h%d>%s</h%d>" % (
            node.level,
            self.html_escape(node.content),
            node.level,
        )
        if self.toc is not None:
            self.toc.add_headline(node.level, node.content)
            # add link attribute for toc navigation
            header = '<a name="%s">%s</a>' % (self.html_escape(node.content), header)

        header += "\n"
        return header

    def preformatted_emit(self, node):
        return "<pre>%s</pre>" % self.html_escape(node.content)

    def link_emit(self, node):
        target = node.content
        if node.children:
            inside = self.emit_children(node)
        else:
            inside = self.html_escape(target)

        return '<a href="%s">%s</a>' % (self.attr_escape(target), inside)

    def image_emit(self, node):
        target = node.content
        text = self.attr_escape(self.get_text(node))

        return '<img src="%s" title="%s" alt="%s" />' % (
            self.attr_escape(target),
            text,
            text,
        )

    def macro_emit(self, node):
        # print(node.debug())
        macro_name = node.macro_name
        text = node.content
        macro = None

        args = node.macro_args
        try:
            macro_kwargs = string2dict(args)
        except ValueError as e:
            exc_info = sys.exc_info()
            return self.error(
                "Wrong macro arguments: %s for macro '%s' (maybe wrong macro tag syntax?)"
                % (repr2(args), macro_name),
                exc_info,
            )

        macro_kwargs["text"] = text

        # Inject the Django request object so it can be used inside the macro
        # processor
        macro_kwargs["view"] = self.view

        exc_info = None
        if isinstance(self.macros, dict):
            try:
                macro = self.macros[macro_name]
            except KeyError as e:
                exc_info = sys.exc_info()
        else:
            try:
                macro = getattr(self.macros, macro_name)
            except AttributeError as e:
                exc_info = sys.exc_info()

        if macro == None:
            return self.error("Macro '%s' doesn't exist" % macro_name, exc_info)

        try:
            result = macro(**macro_kwargs)
        except TypeError as err:
            msg = "Macro '%s' error: %s" % (macro_name, err)
            exc_info = sys.exc_info()
            if self.verbose > 1:
                if self.verbose > 2:
                    raise

                # Inject more information about the macro in traceback
                etype, evalue, etb = exc_info
                import inspect

                try:
                    filename = inspect.getfile(macro)
                except TypeError:
                    pass
                else:
                    try:
                        sourceline = inspect.getsourcelines(macro)[0][0].strip()
                    except IOError as err:
                        evalue = etype(
                            "%s (error getting sourceline: %s from %s)"
                            % (evalue, err, filename)
                        )
                    else:
                        evalue = etype(
                            "%s (sourceline: %r from %s)"
                            % (evalue, sourceline, filename)
                        )
                    exc_info = etype, evalue, etb

            return self.error(msg, exc_info)
        except Exception as err:
            return self.error(
                "Macro '%s' error: %s" % (macro_name, err), exc_info=sys.exc_info()
            )

        if not isinstance(result, TEXT_TYPE):
            msg = "Macro '%s' doesn't return a unicode string!" % macro_name
            if self.verbose > 1:
                msg += " - returns: %r, type %r" % (result, type(result))
            return self.error(msg)

        if node.kind == "macro_block":
            result += "\n"

        return result

    macro_inline_emit = macro_emit
    macro_block_emit = macro_emit

    def break_emit(self, node):
        if node.parent.kind == "list_item":
            return "<br />\n" + "\t" * node.parent.level
        elif node.parent.kind in ("table_head", "table_cell"):
            return "<br />\n\t\t"
        else:
            return "<br />\n"

    def line_emit(self, node):
        return "\n"

    def pre_block_emit(self, node):
        """ pre block, with newline at the end """
        return "<pre>%s</pre>\n" % self.html_escape(node.content)

    def pre_inline_emit(self, node):
        """ pre without newline at the end """
        return "<tt>%s</tt>" % self.html_escape(node.content)

    def default_emit(self, node):
        """Fallback function for emitting unknown nodes."""
        raise NotImplementedError("Node '%s' unknown" % node.kind)

    def emit_children(self, node):
        """Emit all the children of a node."""
        return "".join([self.emit_node(child) for child in node.children])

    def emit_node(self, node):
        """Emit a single node."""
        # print("%s_emit: %r" % (node.kind, node.content))
        emit = getattr(self, "%s_emit" % node.kind, self.default_emit)
        return emit(node)

    def emit(self):
        """Emit the document represented by self.root DOM tree."""
        document = self.emit_node(self.root).strip()
        if self.toc is not None:
            return self.toc.emit(document)
        else:
            return document

    def error(self, text, exc_info=None):
        """
        Error Handling.
        """
        if self.verbose > 1 and exc_info:
            exc_type, exc_value, exc_traceback = exc_info
            exception = "".join(
                traceback.format_exception(exc_type, exc_value, exc_traceback)
            )
            self.stderr.write(exception)

        if self.verbose > 0:
            return "[Error: %s]\n" % text
        else:
            # No error output
            return ""


if __name__ == "__main__":
    txt = """Local test
<<toc>>
= headline 1 level 1
== headline 2 level 2
== headline 3 level 2
==== headline 4 level 4
= headline 5 level 1
=== headline 6 level 3
"""

    print("-" * 80)
    #    from mycms.creole_alt.creole import CreoleParser
    p = CreoleParser(txt)
    document = p.parse()
    p.debug()

    html = HtmlEmitter(document, verbose=999).emit()
    print(html)
    print("-" * 79)
    print(html.replace(" ", ".").replace("\n", "\\n\n"))



# coding: utf-8


"""
    python creole utilities
    ~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2011-2014 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import shlex

from mycms.creole.py3compat import TEXT_TYPE, PY3, repr2

try:
    from pygments import lexers
    from pygments.formatters import HtmlFormatter

    PYGMENTS = True
except ImportError:
    PYGMENTS = False


# For string2dict()
KEYWORD_MAP = {
    "True": True,
    "False": False,
    "None": None,
}


def string2dict(raw_content, encoding="utf-8"):
    """
    convert a string into a dictionary. e.g.:

    >>> string2dict('key="value"')
    {'key': 'value'}

    >>> string2dict('key1="value1" key2="value2"') == {'key2': 'value2', 'key1': 'value1'}
    True

    See test_creole2html.TestString2Dict()
    """
    if not PY3 and isinstance(raw_content, TEXT_TYPE):
        # shlex.split doesn't work with unicode?!?
        raw_content = raw_content.encode(encoding)

    parts = shlex.split(raw_content)

    result = {}
    for part in parts:
        key, value = part.split("=", 1)

        if value in KEYWORD_MAP:
            # True False or None
            value = KEYWORD_MAP[value]
        else:
            # A number?
            try:
                value = int(value.strip("'\""))
            except ValueError:
                pass

        result[key] = value

    return result


def dict2string(d):
    """
    FIXME: Find a better was to do this.

    >>> dict2string({'foo':"bar", "no":123})
    "foo='bar' no=123"

    >>> dict2string({"foo":'bar', "no":"ABC"})
    "foo='bar' no='ABC'"

    See test_creole2html.TestDict2String()
    """
    attr_list = []
    for key, value in sorted(d.items()):
        value_string = repr2(value)
        attr_list.append("%s=%s" % (key, value_string))
    return " ".join(attr_list)


def get_pygments_formatter():
    if PYGMENTS:
        return HtmlFormatter(
            lineos=True,
            encoding="utf-8",
            style="colorful",
            outencoding="utf-8",
            cssclass="pygments",
        )


def get_pygments_lexer(source_type, code):
    if PYGMENTS:
        try:
            return lexers.get_lexer_by_name(source_type)
        except:
            return lexers.guess_lexer(code)
    else:
        return None


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())



# coding: utf-8


"""
    Creole wiki markup parser

    See http://wikicreole.org/ for latest specs.

    Notes:
    * No markup allowed in headings.
      Creole 1.0 does not require us to support this.
    * No markup allowed in table headings.
      Creole 1.0 does not require us to support this.
    * No (non-bracketed) generic url recognition: this is "mission impossible"
      except if you want to risk lots of false positives. Only known protocols
      are recognized.
    * We do not allow ":" before "//" italic markup to avoid urls with
      unrecognized schemes (like wtf://server/path) triggering italic rendering
      for the rest of the paragraph.

    :copyleft: 2008-2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from __future__ import division, absolute_import, print_function, unicode_literals

import re

from mycms.creole.creole2html.rules import (
    BlockRules,
    INLINE_FLAGS,
    INLINE_RULES,
    SpecialRules,
    InlineRules,
)
from mycms.creole.py3compat import TEXT_TYPE
from mycms.creole.shared.document_tree import DocNode, DebugList


class CreoleParser(object):
    """
    Parse the raw text and create a document object
    that can be converted into output using Emitter.
    """

    # For pre escaping, in creole 1.0 done with ~:
    pre_escape_re = re.compile(
        SpecialRules.pre_escape, re.MULTILINE | re.VERBOSE | re.UNICODE
    )

    # for link descriptions:
    link_re = re.compile(
        "|".join([InlineRules.image, InlineRules.linebreak, InlineRules.char]),
        re.VERBOSE | re.UNICODE,
    )
    # for list items:
    item_re = re.compile(SpecialRules.item, re.VERBOSE | re.UNICODE | re.MULTILINE)

    # for table cells:
    cell_re = re.compile(SpecialRules.cell, re.VERBOSE | re.UNICODE)

    # For inline elements:
    inline_re = re.compile("|".join(INLINE_RULES), INLINE_FLAGS)

    def __init__(self, raw, block_rules=None, blog_line_breaks=True):
        assert isinstance(raw, TEXT_TYPE)
        self.raw = raw

        if block_rules is None:
            block_rules = BlockRules(blog_line_breaks=blog_line_breaks)

        # setup block element rules:
        self.block_re = re.compile("|".join(block_rules.rules), block_rules.re_flags)

        self.blog_line_breaks = blog_line_breaks

        self.root = DocNode("document", None)
        self.cur = self.root  # The most recent document node
        self.text = None  # The node to add inline characters to
        self.last_text_break = None  # Last break node, inserted by _text_repl()

        # Filled with all macros that's in the text
        self.root.used_macros = set()

    # --------------------------------------------------------------------------

    def cleanup_break(self, old_cur):
        """
        remove unused end line breaks.
        Should be called before a new block element.
        e.g.:
          <p>line one<br />
          line two<br />     <--- remove this br-tag
          </p>
        """
        if self.cur.children:
            last_child = self.cur.children[-1]
            if last_child.kind == "break":
                del self.cur.children[-1]

    def _upto(self, node, kinds):
        """
        Look up the tree to the first occurence
        of one of the listed kinds of nodes or root.
        Start at the node node.
        """
        self.cleanup_break(node)  # remove unused end line breaks.
        while node.parent is not None and not node.kind in kinds:
            node = node.parent

        return node

    def _upto_block(self):
        self.cur = self._upto(self.cur, ("document",))  # 'section', 'blockquote'))

    # __________________________________________________________________________
    # The _*_repl methods called for matches in regexps. Sometimes the
    # same method needs several names, because of group names in regexps.

    def _text_repl(self, groups):
        #        print("_text_repl()", self.cur.kind)
        #        self.debug_groups(groups)

        if self.cur.kind in ("table", "table_row", "bullet_list", "number_list"):
            self._upto_block()

        if self.cur.kind in ("document", "section", "blockquote"):
            self.cur = DocNode("paragraph", self.cur)

        text = groups.get("text", "")

        if groups.get("space"):
            # use wikipedia style line breaks and seperate a new line with one space
            text = " " + text

        self.parse_inline(text)

        if groups.get("break") and self.cur.kind in (
            "paragraph",
            "emphasis",
            "strong",
            "pre_inline",
        ):
            self.last_text_break = DocNode("break", self.cur, "")

        self.text = None

    _break_repl = _text_repl
    _space_repl = _text_repl

    def _url_repl(self, groups):
        """Handle raw urls in text."""
        if not groups.get("escaped_url"):
            # this url is NOT escaped
            target = groups.get("url_target", "")
            node = DocNode("link", self.cur)
            node.content = target
            DocNode("text", node, node.content)
            self.text = None
        else:
            # this url is escaped, we render it as text
            if self.text is None:
                self.text = DocNode("text", self.cur, "")
            self.text.content += groups.get("url_target")

    _url_target_repl = _url_repl
    _url_proto_repl = _url_repl
    _escaped_url_repl = _url_repl

    def _link_repl(self, groups):
        """Handle all kinds of links."""
        target = groups.get("link_target", "")
        text = (groups.get("link_text", "") or "").strip()
        parent = self.cur
        self.cur = DocNode("link", self.cur)
        self.cur.content = target
        self.text = None
        re.sub(self.link_re, self._replace, text)
        self.cur = parent
        self.text = None

    _link_target_repl = _link_repl
    _link_text_repl = _link_repl

    # --------------------------------------------------------------------------

    def _add_macro(self, groups, macro_type, name_key, args_key, text_key=None):
        """
        generic method to handle the macro, used for all variants:
        inline, inline-tag, block
        """
        # self.debug_groups(groups)
        assert macro_type in ("macro_inline", "macro_block")

        if text_key:
            macro_text = groups.get(text_key, "").strip()
        else:
            macro_text = None

        node = DocNode(macro_type, self.cur, macro_text)
        macro_name = groups[name_key]
        node.macro_name = macro_name
        self.root.used_macros.add(macro_name)
        node.macro_args = groups.get(args_key, "").strip()

        self.text = None

    def _macro_block_repl(self, groups):
        """
        block macro, e.g:
        <<macro args="foo">>
        some
        lines
        <</macro>>
        """
        self._upto_block()
        self.cur = self.root
        self._add_macro(
            groups,
            macro_type="macro_block",
            name_key="macro_block_start",
            args_key="macro_block_args",
            text_key="macro_block_text",
        )

    _macro_block_start_repl = _macro_block_repl
    _macro_block_args_repl = _macro_block_repl
    _macro_block_text_repl = _macro_block_repl

    def _macro_tag_repl(self, groups):
        """
        A single macro tag, e.g.: <<macro-a foo="bar">> or <<macro />>
        """
        self._add_macro(
            groups,
            macro_type="macro_inline",
            name_key="macro_tag_name",
            args_key="macro_tag_args",
            text_key=None,
        )

    _macro_tag_name_repl = _macro_tag_repl
    _macro_tag_args_repl = _macro_tag_repl

    def _macro_inline_repl(self, groups):
        """
        inline macro tag with data, e.g.: <<macro>>text<</macro>>
        """
        self._add_macro(
            groups,
            macro_type="macro_inline",
            name_key="macro_inline_start",
            args_key="macro_inline_args",
            text_key="macro_inline_text",
        )

    _macro_inline_start_repl = _macro_inline_repl
    _macro_inline_args_repl = _macro_inline_repl
    _macro_inline_text_repl = _macro_inline_repl

    # --------------------------------------------------------------------------

    def _image_repl(self, groups):
        """Handles images and attachemnts included in the page."""
        target = groups.get("image_target", "").strip()
        text = (groups.get("image_text", "") or "").strip()
        node = DocNode("image", self.cur, target)
        DocNode("text", node, text or node.content)
        self.text = None

    _image_target_repl = _image_repl
    _image_text_repl = _image_repl

    def _separator_repl(self, groups):
        self._upto_block()
        DocNode("separator", self.cur)

    def _item_repl(self, groups):
        """ List item """
        bullet = groups.get("item_head", "")
        text = groups.get("item_text", "")
        if bullet[-1] == "#":
            kind = "number_list"
        else:
            kind = "bullet_list"
        level = len(bullet) - 1
        lst = self.cur
        # Find a list of the same kind and level up the tree
        while (
            lst
            and not (lst.kind in ("number_list", "bullet_list") and lst.level == level)
            and not lst.kind in ("document", "section", "blockquote")
        ):
            lst = lst.parent
        if lst and lst.kind == kind:
            self.cur = lst
        else:
            # Create a new level of list
            self.cur = self._upto(
                self.cur, ("list_item", "document", "section", "blockquote")
            )
            self.cur = DocNode(kind, self.cur)
            self.cur.level = level
        self.cur = DocNode("list_item", self.cur)
        self.cur.level = level + 1
        self.parse_inline(text)
        self.text = None

    _item_text_repl = _item_repl
    _item_head_repl = _item_repl

    def _list_repl(self, groups):
        """ complete list """
        self.item_re.sub(self._replace, groups["list"])

    def _head_repl(self, groups):
        self._upto_block()
        node = DocNode("header", self.cur, groups["head_text"].strip())
        node.level = len(groups["head_head"])
        self.text = None

    _head_head_repl = _head_repl
    _head_text_repl = _head_repl

    def _table_repl(self, groups):
        row = groups.get("table", "|").strip()
        self.cur = self._upto(self.cur, ("table", "document", "section", "blockquote"))
        if self.cur.kind != "table":
            self.cur = DocNode("table", self.cur)
        tb = self.cur
        tr = DocNode("table_row", tb)

        for m in self.cell_re.finditer(row):
            cell = m.group("cell")
            if cell:
                text = cell.strip()
                self.cur = DocNode("table_cell", tr)
                self.text = None
            else:
                text = m.group("head").strip("= ")
                self.cur = DocNode("table_head", tr)
                self.text = DocNode("text", self.cur, "")
            self.parse_inline(text)

        self.cur = tb
        self.text = None

    def _pre_block_repl(self, groups):
        self._upto_block()
        kind = groups.get("pre_block_kind", None)
        text = groups.get("pre_block_text", "")

        def remove_tilde(m):
            return m.group("indent") + m.group("rest")

        text = self.pre_escape_re.sub(remove_tilde, text)
        node = DocNode("pre_block", self.cur, text)
        node.sect = kind or ""
        self.text = None

    _pre_block_text_repl = _pre_block_repl
    _pre_block_head_repl = _pre_block_repl
    _pre_block_kind_repl = _pre_block_repl

    def _line_repl(self, groups):
        """ Transfer newline from the original markup into the html code """
        self._upto_block()
        DocNode("line", self.cur, "")

    def _pre_inline_repl(self, groups):
        text = groups.get("pre_inline_text", "")
        DocNode("pre_inline", self.cur, text)
        self.text = None

    _pre_inline_text_repl = _pre_inline_repl
    _pre_inline_head_repl = _pre_inline_repl

    # --------------------------------------------------------------------------

    def _inline_mark(self, groups, key):
        self.cur = DocNode(key, self.cur)

        self.text = None
        text = groups["%s_text" % key]
        self.parse_inline(text)

        self.cur = self._upto(self.cur, (key,)).parent
        self.text = None

    # TODO: How can we generalize that:
    def _emphasis_repl(self, groups):
        self._inline_mark(groups, key="emphasis")

    _emphasis_text_repl = _emphasis_repl

    def _strong_repl(self, groups):
        self._inline_mark(groups, key="strong")

    _strong_text_repl = _strong_repl

    def _monospace_repl(self, groups):
        self._inline_mark(groups, key="monospace")

    _monospace_text_repl = _monospace_repl

    def _superscript_repl(self, groups):
        self._inline_mark(groups, key="superscript")

    _superscript_text_repl = _superscript_repl

    def _subscript_repl(self, groups):
        self._inline_mark(groups, key="subscript")

    _subscript_text_repl = _subscript_repl

    def _underline_repl(self, groups):
        self._inline_mark(groups, key="underline")

    _underline_text_repl = _underline_repl

    def _small_repl(self, groups):
        self._inline_mark(groups, key="small")

    _small_text_repl = _small_repl

    def _delete_repl(self, groups):
        self._inline_mark(groups, key="delete")

    _delete_text_repl = _delete_repl

    # --------------------------------------------------------------------------

    def _linebreak_repl(self, groups):
        DocNode("break", self.cur, None)
        self.text = None

    def _escape_repl(self, groups):
        if self.text is None:
            self.text = DocNode("text", self.cur, "")
        self.text.content += groups.get("escaped_char", "")

    _escaped_char_repl = _escape_repl

    def _char_repl(self, groups):
        if self.text is None:
            self.text = DocNode("text", self.cur, "")
        self.text.content += groups.get("char", "")

    # --------------------------------------------------------------------------

    def _replace(self, match):
        """Invoke appropriate _*_repl method. Called for every matched group."""

        #        def debug(groups):
        #            from pprint import pformat
        #            data = dict([
        #                group for group in groups.items() if group[1] is not None
        #            ])
        #            print("%s\n" % pformat(data))

        groups = match.groupdict()
        for name, text in groups.items():
            if text is not None:
                # if name != "char": debug(groups)
                replace_method = getattr(self, "_%s_repl" % name)
                replace_method(groups)
                return

    def parse_inline(self, raw):
        """Recognize inline elements inside blocks."""
        re.sub(self.inline_re, self._replace, raw)

    def parse_block(self, raw):
        """Recognize block elements."""
        re.sub(self.block_re, self._replace, raw)

    def parse(self):
        """Parse the text given as self.raw and return DOM tree."""
        # convert all lineendings to \n
        text = self.raw.replace("\r\n", "\n").replace("\r", "\n")
        self.parse_block(text)
        return self.root

    # --------------------------------------------------------------------------
    def debug(self, start_node=None):
        """
        Display the current document tree
        """
        print("_" * 80)

        if start_node == None:
            start_node = self.root
            print("  document tree:")
        else:
            print("  tree from %s:" % start_node)

        print("=" * 80)

        def emit(node, ident=0):
            for child in node.children:
                print("%s%s: %r" % (" " * ident, child.kind, child.content))
                emit(child, ident + 4)

        emit(start_node)
        print("*" * 80)

    def debug_groups(self, groups):
        print("_" * 80)
        print("  debug groups:")
        for name, text in groups.items():
            if text is not None:
                print("%15s: %r" % (name, text))
        print("-" * 80)


if __name__ == "__main__":
    import doctest

    print(doctest.testmod())

    print("-" * 80)

    txt = """A <<test_macro1 args="foo1">>bar1<</test_macro1>> in a line..."""

    print(txt)
    print("-" * 80)

    blog_line_breaks = False

    p = CreoleParser(txt, blog_line_breaks=blog_line_breaks)
    document = p.parse()
    p.debug()

    def display_match(match):
        groups = match.groupdict()
        for name, text in groups.items():
            if name != "char" and text != None:
                print("%20s: %r" % (name, text))

    parser = CreoleParser("", blog_line_breaks=blog_line_breaks)

    print("_" * 80)
    print("merged block rules test:")
    re.sub(parser.block_re, display_match, txt)

    print("_" * 80)
    print("merged inline rules test:")
    re.sub(parser.inline_re, display_match, txt)

    def test_single(rules, flags, txt):
        for rule in rules:
            rexp = re.compile(rule, flags)
            rexp.sub(display_match, txt)

    print("_" * 80)
    print("single block rules match test:")
    block_rules = BlockRules()
    test_single(block_rules.rules, block_rules.re_flags, txt)

    print("_" * 80)
    print("single inline rules match test:")
    test_single(INLINE_RULES, INLINE_FLAGS, txt)

    print("---END---")



from mycms.models import CMSEntries
from mycms.models import CMSPageTypes
from mycms.view_handlers.mycms_view import ViewObject
from mycms.view_handlers.mycms_view import ArticleList

from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.db.utils import OperationalError

import logging

logger = logging.getLogger("mycms.page_handlers")


# #####################
# We ensure that the base page types are already created.
# #####################

from mycms.view_handlers import page_types


class AllArticlesPage(page_types.BasePage):
    def __init__(self, page_object, request=None):
        self.page_object = page_object
        self.request = request
        self._article_list = None

    # TODO: Refactor the code so that there is only one articles() method used
    # by all and just control by a flag whether we want to recursively or not.

    def articles(self):

        """Here we load all pages that says we are their parent."""

        from django.db.models import Q

        # ######################################################################
        # This loads up all the articles that are
        # of type single_page or multipage and has the provided parent.

        limit, offset, page = self.get_list_params()

        if (
            (self._article_list is None)
            or (limit != self._article_list.limit)
            or (offset != self._article_list.offset)
            or (page != self._article_list.page)
        ):

            # the above "if" works because the first boolean tests if
            # self._article_list is None and the rest are not evaluated further.

            # This is a lame attempt at ensuring that multple calls to article
            # is avoided by caching the results so that when processing the
            # template we do not need to execute this code any further.

            offset = page * limit - limit

            # Get the results via offset and limit.
            obj_list = CMSEntries.objects.filter(
                (
                    Q(page_type=page_types.singlepageview_pagetype_obj)
                    | Q(page_type=page_types.multipageview_pagetype_obj)
                )
                & Q(lists_include=True)
                & Q(published=True)
            )[offset : offset + limit]

            num_entries = CMSEntries.objects.filter(
                (
                    Q(page_type=page_types.singlepageview_pagetype_obj)
                    | Q(page_type=page_types.multipageview_pagetype_obj)
                )
                & Q(lists_include=True)
                & Q(published=True)
            )[offset : offset + limit].count()

            num_total_cmsentries = CMSEntries.objects.filter(
                (
                    Q(page_type=page_types.singlepageview_pagetype_obj)
                    | Q(page_type=page_types.multipageview_pagetype_obj)
                )
                & Q(lists_include=True)
                & Q(published=True)
            ).count()

            self._article_list = ArticleList(obj_list, num_total_cmsentries, page)
            self._article_list.limit = limit
            self._article_list.offset = offset
            self._article_list.num_total_cmsentries = num_total_cmsentries

            for obj in obj_list:
                self._article_list.append(ViewObject(page_object=obj))

        return self._article_list

    def get_categories(self):
        """Returns a list of all child categories of type: CATEGORY"""

        obj_list = CMSEntries.objects.filter(
            path__path__parent__id=self.page_object.id, page_type=page_obj.page_type
        )

        return obj_list

    def page_types(self):

        """
        Refactor me into a parent class.
        returns a list fo page_types
        """

        pagetype_objs = CMSPageTypes.objects.all()

        return pagetype_objs

    def on_create(self):
        pass



from django import template
from django.core.exceptions import ObjectDoesNotExist
from django.template import Template, Context
from django.template.loader import get_template
from django.contrib.auth.models import User
from django.core.paginator import Paginator


from django.template.loader import render_to_string

# from mycms.models import Paths, Pages
# from mycms.pageview.base import get_pageview

from mycms.models import CMSPaths
from mycms.models import CMSEntries

register = template.Library()


@register.inclusion_tag("mycms/templatetags/widgets/categories_menu.html")
def categories_menu():
    """
    Used to get a full category tree starting from /cms/. 
    """

    try:
        parent = CMSEntries.objects.get(path__path="/")
    except ObjectDoesNotExist as e:
        msg = (
            "No CMSEntries to produce full_menu from /. Perhaps / does not yet exist!!"
        )
        logger.fatal(msg)
        parent = None
    return {"parent": parent}


@register.inclusion_tag("mycms/templatetags/widgets/google_adds.html")
def google_adds():

    return {
        "width": 300,
        "height": 600,
        "add-client": "data=ca-pub-9449210019187312",
        "data-ad-slot": "6783855847",
    }


@register.inclusion_tag("mycms/templatetags/widgets/search.html")
def search():
    return {}


@register.inclusion_tag("mycms/templatetags/widgets/google_adds.html")
def addsbygoogle():

    return {
        "width": 300,
        "height": 600,
        "add-client": "data=ca-pub-9449210019187312",
        "data-ad-slot": "6783855847",
        "data-ad-format": "auto",
        "data-full-width-responsive": "true",
    }



from django.core.management.base import BaseCommand, CommandError
from mycms.models import CMSEntries


from django.core.exceptions import ObjectDoesNotExist
from django.core.exceptions import MultipleObjectsReturned

from mycms.view_handlers.page_types import singlepageview_pagetype_obj
from mycms.view_handlers.page_types import multipageview_pagetype_obj
from mycms.view_handlers.page_types import allarticles_pagetype_obj


class Command(BaseCommand):
    help = "Closes the specified poll for voting"

    def handle(self, *args, **options):

        from django.db.models import Q

        c = CMSEntries.objects.get(path__path="/sysadmin/linux")

        print(c)
        obj_list = CMSEntries.objects.filter(
            (
                Q(page_type=singlepageview_pagetype_obj)
                | Q(page_type=multipageview_pagetype_obj)
            )
            & Q(path__path__startswith=c.path.path)
        )

        for obj in obj_list:
            print(obj.path)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-04-11 19:02
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="CMSContents",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(blank=True, max_length=1024, null=True)),
                ("content", models.TextField(default="Empty", max_length=20480)),
                ("timestamp", models.DateTimeField(auto_now=True)),
                (
                    "meta_description",
                    models.TextField(blank=True, default="", max_length=20480),
                ),
                ("page", models.IntegerField(default=1)),
            ],
        ),
        migrations.CreateModel(
            name="CMSEntries",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(default=None, max_length=1024)),
                ("slug", models.SlugField(max_length=1024, unique=True)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_modified", models.DateTimeField(auto_now_add=True)),
                ("frontpage", models.BooleanField(default=False)),
                ("published", models.BooleanField(default=False)),
                ("page_number", models.IntegerField(default=1)),
                ("content", models.ManyToManyField(blank=True, to="mycms.CMSContents")),
                (
                    "created_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="CMSMarkUps",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("markup", models.CharField(default="Creole", max_length=128)),
            ],
        ),
        migrations.CreateModel(
            name="CMSPageTypes",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("page_type", models.CharField(default="DefaultType", max_length=64)),
                ("text", models.CharField(default="default class", max_length=128)),
                ("view_class", models.CharField(default="DefaultView", max_length=256)),
                ("view_template", models.CharField(default=None, max_length=32)),
            ],
        ),
        migrations.CreateModel(
            name="CMSPaths",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("path", models.CharField(max_length=2000, null=True)),
                (
                    "parent",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to="mycms.CMSPaths",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="CMSTags",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(default="NotSet", max_length=256)),
            ],
        ),
        migrations.CreateModel(
            name="CMSTemplates",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(default="page.html", max_length=1024)),
                (
                    "template",
                    models.TextField(default="empty template", max_length=10240),
                ),
            ],
        ),
        migrations.AddField(
            model_name="cmsentries",
            name="page_type",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to="mycms.CMSPageTypes",
            ),
        ),
        migrations.AddField(
            model_name="cmsentries",
            name="path",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to="mycms.CMSPaths",
            ),
        ),
        migrations.AddField(
            model_name="cmsentries",
            name="template",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to="mycms.CMSTemplates",
            ),
        ),
        migrations.AddField(
            model_name="cmscontents",
            name="markup",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to="mycms.CMSMarkUps",
            ),
        ),
        migrations.AddField(
            model_name="cmscontents",
            name="tags",
            field=models.ManyToManyField(blank=True, to="mycms.CMSTags"),
        ),
    ]



#########################################################################
""" wingdbstub.py    -- Debug stub for debuggifying Python programs

Copyright (c) 1999-2001, Archaeopteryx Software, Inc.  All rights reserved.

Written by Stephan R.A. Deibel and John P. Ehresman

Usage:
-----

This is the file that Wing DB users copy into their python project 
directory if they want to be able to debug programs that are launched
outside of the IDE (e.g., CGI scripts, in response to a browser page
load).

To use this, edit the configuration values below to match your 
Wing IDE installation and requirements of your project.

Then, add the following line to your code:

  import wingdbstub

Debugging will start immediately after this import statements.

Next make sure that your IDE is running and that it's configured to accept
connections from the host the debug program will be running on.

Now, invoking your python file should run the code within the debugger.
Note, however, that Wing will not stop in the code unless a breakpoint
is set.

If the debug process is started before the IDE, or is not listening
at the time this module is imported then the program will run with
debugging until an attach request is seen.  Attaching only works 
if the .wingdebugpw file is present; see the manual for details.

One win32, you either need to edit WINGHOME in this script or
pass in an environment variable called WINGHOME that points to
the Wing IDE installation directory.

"""
#########################################################################

import sys
import os
import imp


# ------------------------------------------------------------------------
# Default configuration values:  Note that the named environment
# variables, if set, will override these settings.

# Set this to 1 to disable all debugging; 0 to enable debugging
# (WINGDB_DISABLED environment variable)
kWingDebugDisabled = 0

# Host:port of the IDE within which to debug: As configured in the IDE
# with the Server Port preference
# (WINGDB_HOSTPORT environment variable)
kWingHostPort = "localhost:50005"

# Port on which to listen for connection requests, so that the
# IDE can (re)attach to the debug process after it has started
# This is only used when the debug process is not attached to
# an IDE or the IDE has dropped its connection. The configured
# port can optionally be added to the IDE's Common Attach Hosts
# preference. Note that a random port is used instead if this
# port is already in use!
# (WINGDB_ATTACHPORT environment variable)
kAttachPort = "50015"

# Set this to a filename to log verbose information about the debugger
# internals to a file.  If the file does not exist, it will be created
# as long as its enclosing directory exists and is writeable.  Use
# "<stderr>" or "<stdout>".  Note that "<stderr>" may cause problems
# on win32 if the debug process is not running in a console.
# (WINGDB_LOGFILE environment variable)
kLogFile = None

# Set to get a tremendous amount of logging from the debugger internals
# (WINGDB_LOGVERYVERBOSE)
kLogVeryVerbose = 0

# Set this to 1 when debugging embedded scripts in an environment that
# creates and reuses a Python instance across multiple script invocations:
# It turns off automatic detection of program quit so that the debug session
# can span multiple script invocations.  When this is turned on, you may
# need to call ProgramQuit() on the debugger object to shut down the
# debugger cleanly when your application exits or discards the Python
# instance.  If multiple Python instances are created in the same run,
# only the first one will be able to debug unless it terminates debug
# and the environment variable WINGDB_ACTIVE is unset before importing
# this module in the second or later Python instance.  See the Wing
# IDE manual for details.
kEmbedded = 0

# Path to search for the debug password file and the name of the file
# to use.  The password file contains the encryption type and connect
# password for all connections to the IDE and must match the wingdebugpw
# file in the profile dir used by the IDE.  Any entry of '$<winguserprofile>'
# is replaced by the wing user profile directory for the user that the
# current process is running as
# (WINGDB_PWFILEPATH environment variable)
kPWFilePath = [os.path.dirname(__file__), "$<winguserprofile>"]
kPWFileName = "wingdebugpw"

# Whether to exit if the debugger fails to run or to connect with an IDE
# for whatever reason
kExitOnFailure = 0

# ------------------------------------------------------------------------
# Find Wing debugger installation location

# Edit this to point to your Wing installation or set to None to use env WINGHOME
# On OS X this must be set to name of the Wing IDE application bundle
# (for example, /Applications/WingIDE.app)
WINGHOME = None

if sys.hexversion >= 0x03000000:

    def has_key(o, key):
        return key in o


else:

    def has_key(o, key):
        return o.has_key(key)


# Check environment:  Must have WINGHOME defined if still == None
if WINGHOME == None:
    if has_key(os.environ, "WINGHOME"):
        WINGHOME = os.environ["WINGHOME"]
    else:
        sys.stdout.write(
            "*******************************************************************\n"
        )
        sys.stdout.write(
            "Error: Could not find Wing installation!  You must set WINGHOME or edit\n"
        )
        sys.stdout.write(
            "wingdbstub.py where indicated to point it to the location where\n"
        )
        sys.stdout.write("Wing IDE is installed.\n")
        sys.exit(1)

# The user settings dir where per-user settings & patches are located.  Will be
# set from environment if left as None
kUserSettingsDir = None
if kUserSettingsDir is None:
    kUserSettingsDir = os.environ.get("WINGDB_USERSETTINGS")


def _FindActualWingHome(winghome):
    """ Find the actual directory to use for winghome.  Needed on OS X
  where the .app directory is the preferred dir to use for WINGHOME and
  .app/Contents/MacOS is accepted for backward compatibility. """

    if sys.platform != "darwin":
        return winghome

    app_dir = None
    if os.path.isdir(winghome):
        if winghome.endswith("/"):
            wo_slash = winghome[:-1]
        else:
            wo_slash = winghome

        if wo_slash.endswith(".app"):
            app_dir = wo_slash
        elif wo_slash.endswith(".app/Contents/MacOS"):
            app_dir = wo_slash[: -len("/Contents/MacOS")]

    if app_dir and os.path.isdir(os.path.join(app_dir, "Contents", "Resources")):
        return os.path.join(app_dir, "Contents", "Resources")

    return winghome


def _ImportWingdb(winghome, user_settings=None):
    """ Find & import wingdb module. """

    try:
        exec_dict = {}
        execfile(os.path.join(winghome, "bin", "_patchsupport.py"), exec_dict)
        find_matching = exec_dict["FindMatching"]
        dir_list = find_matching("bin", winghome, user_settings)
    except Exception:
        dir_list = []
    dir_list.extend([os.path.join(winghome, "bin"), os.path.join(winghome, "src")])
    for path in dir_list:
        try:
            f, p, d = imp.find_module("wingdb", [path])
            try:
                return imp.load_module("wingdb", f, p, d)
            finally:
                if f is not None:
                    f.close()
            break
        except ImportError:
            pass


# ------------------------------------------------------------------------
# Set debugger if it hasn't been set -- this is to handle module reloading
# In the reload case, the debugger variable will be set to something
try:
    debugger
except NameError:
    debugger = None

# Start debugging if not disabled and this module has never been imported
# before
if (
    not kWingDebugDisabled
    and debugger is None
    and not has_key(os.environ, "WINGDB_DISABLED")
    and not has_key(os.environ, "WINGDB_ACTIVE")
):

    exit_on_fail = 0

    try:
        # Obtain exit if fails value
        exit_on_fail = os.environ.get("WINGDB_EXITONFAILURE", kExitOnFailure)

        # Obtain configuration for log file to use, if any
        logfile = os.environ.get("WINGDB_LOGFILE", kLogFile)
        if logfile == "-" or logfile == None or len(logfile.strip()) == 0:
            logfile = None

        very_verbose_log = os.environ.get("WINGDB_LOGVERYVERBOSE", kLogVeryVerbose)
        if type(very_verbose_log) == type("") and very_verbose_log.strip() == "":
            very_verbose_log = 0

        # Determine remote host/port where the IDE is running
        hostport = os.environ.get("WINGDB_HOSTPORT", kWingHostPort)
        colonpos = hostport.find(":")
        host = hostport[:colonpos]
        port = int(hostport[colonpos + 1 :])

        # Determine port to listen on locally for attach requests
        attachport = int(os.environ.get("WINGDB_ATTACHPORT", kAttachPort))

        # Check if running embedded script
        embedded = int(os.environ.get("WINGDB_EMBEDDED", kEmbedded))

        # Obtain debug password file search path
        if has_key(os.environ, "WINGDB_PWFILEPATH"):
            pwfile_path = os.environ["WINGDB_PWFILEPATH"].split(os.pathsep)
        else:
            pwfile_path = kPWFilePath

        # Obtain debug password file name
        if has_key(os.environ, "WINGDB_PWFILENAME"):
            pwfile_name = os.environ["WINGDB_PWFILENAME"]
        else:
            pwfile_name = kPWFileName

        # Load wingdb.py
        actual_winghome = _FindActualWingHome(WINGHOME)
        wingdb = _ImportWingdb(actual_winghome, kUserSettingsDir)
        if wingdb == None:
            sys.stdout.write(
                "*******************************************************************\n"
            )
            sys.stdout.write(
                "Error: Cannot find wingdb.py in $(WINGHOME)/bin or $(WINGHOME)/src\n"
            )
            sys.stdout.write(
                "Error: Please check the WINGHOME definition in wingdbstub.py\n"
            )
            sys.exit(2)

        # Find the netserver module and create an error stream
        netserver = wingdb.FindNetServerModule(actual_winghome, kUserSettingsDir)
        err = wingdb.CreateErrStream(netserver, logfile, very_verbose_log)

        # Start debugging
        debugger = netserver.CNetworkServer(
            host,
            port,
            attachport,
            err,
            pwfile_path=pwfile_path,
            pwfile_name=pwfile_name,
            autoquit=not embedded,
        )
        debugger.StartDebug(stophere=0)
        os.environ["WINGDB_ACTIVE"] = "1"
        if debugger.ChannelClosed():
            raise ValueError("Not connected")

    except:
        if exit_on_fail:
            raise
        else:
            pass


def Ensure(require_connection=1, require_debugger=1):
    """ Ensure the debugger is started and attempt to connect to the IDE if
  not already connected.  Will raise a ValueError if:
  
  * the require_connection arg is true and the debugger is unable to connect
  * the require_debugger arg is true and the debugger cannot be loaded
  
  If SuspendDebug() has been called through the low-level API, calling
  Ensure() resets the suspend count to zero and additional calls to
  ResumeDebug() will be ignored until SuspendDebug() is called again.
  
  """

    if debugger is None:
        if require_debugger:
            raise ValueError("No debugger")
        return

    resumed = debugger.ResumeDebug()
    while resumed > 0:
        resumed = debugger.ResumeDebug()

    if not debugger.DebugActive():
        debugger.StartDebug()
    elif debugger.ChannelClosed():
        debugger.ConnectToClient()

    if require_connection and debugger.ChannelClosed():
        raise ValueError("Not connected")



import datetime
from haystack import indexes
from mycms.models import CMSEntries


class CMSEntriesIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(document=True, use_template=True)
    author = indexes.CharField(model_attr="created_by")
    pub_date = indexes.DateTimeField(model_attr="date_modified")

    def get_model(self):
        return CMSEntries

    def index_queryset(self, using=None):
        return self.get_model().objects.filter(published=True)



from time import strptime, strftime
from django import forms
from django.db import models
from django.forms import fields
from conflux.widgets import JqSplitDateTimeWidget


class JqSplitDateTimeField(fields.MultiValueField):
    widget = JqSplitDateTimeWidget

    def __init__(self, *args, **kwargs):
        """
        Have to pass a list of field types to the constructor, else we
        won't get any data to our compress method.
        """
        all_fields = (
            fields.CharField(max_length=10),
            fields.CharField(max_length=2),
            fields.CharField(max_length=2),
            fields.ChoiceField(choices=[("AM", "AM"), ("PM", "PM")]),
        )
        super(JqSplitDateTimeField, self).__init__(all_fields, *args, **kwargs)

    def compress(self, data_list):
        """
        Takes the values from the MultiWidget and passes them as a
        list to this function. This function needs to compress the
        list into a single object to save.
        """
        if data_list:
            if not (data_list[0] and data_list[1] and data_list[2] and data_list[3]):
                raise forms.ValidationError("Field is missing data.")
            input_time = strptime(
                "%s:%s %s" % (data_list[1], data_list[2], data_list[3]), "%I:%M %p"
            )
            datetime_string = "%s %s" % (data_list[0], strftime("%H:%M", input_time))
            print("Datetime: %s" % datetime_string)
            return datetime_string
        return None
